from cliche import cli
from pydantic import BaseModel


class B(BaseModel):
    b: str


class Item(BaseModel):
    a: str = "good_one"
    b: str


@cli
def print_item(item: Item, b: int = 2):
    print(repr(item), b)



from cliche import List, cli, Set


@cli
def a(z=[1, 1]):
    """ Test default list default value """
    print(z)


@cli
def b(z: List[int] = [1, 1]):
    """ Test list typing and list default value """
    print(z)


@cli
def c(z: List[int]):
    """ Test list typing """
    print(z)


@cli
def d(z: List[str]):
    """ Test list typing """
    print(z)


# slightly broken
# (feb2018) pascal@archbook:/home/.../cliche/examples$ cliche list_of_items.py f 1 2
# [1, 2] None


@cli
def e(a: int = 1, z: List[int] = None):
    """ Test list typing with None as default """
    print(a, z)


@cli
def f(a: int, z: List[int] = None):
    """ Test pos argument and list typing with None as default """
    print(a, z)


@cli
def g(z: Set[str]):
    """ Test set default """
    print(z)


@cli
def h(z: Set[str] = set(["a"])):
    """ Test set default """
    print(z)



""" Base tests"""
import pytest
import cliche
from cliche import cli, main
from typing import List


def mainer(*args):
    cliche.main_called = []
    main(None, None, "simple", *args)


@cli
def basic():
    pass


def test_basic_int_add():
    expected = 3

    @cli
    def simple(first: int, second: float):
        assert first + second == expected

    mainer("1", "2")


def test_basic_docs():
    expected = 3

    @cli
    def simple(first: int, second: float):
        """Explanation

        :param first: First
        :param second: Second
        """
        assert first + second == expected

    mainer("1", "2")


def test_kw():
    @cli
    def simple(first: int = 1):
        assert first == 3

    mainer("--first", "3")
    mainer("-f", "3")


def test_basic_default():
    expected = 3

    @cli
    def simple(first: int, second: float = 2):
        """Explanation

        :param first: First
        :param second: Second
        """
        assert first + second == expected

    mainer("1")


def test_empty_list():
    @cli
    def simple(first: List[str]):
        """Explanation

        :param first: First
        """
        assert first == []

    mainer()


def test_list_int():
    @cli
    def simple(first: List[int]):
        """Explanation

        :param first: First
        """
        assert first == [1]

    mainer("1")



{{first_line}}

# the above should be dynamic
import os
import sys
import re
import time
import json
import glob

sys.cliche_loaded_modules__ = set(sys.modules)
sys.cliche_ts__ = time.time()
use_timing = "--timing" in sys.argv

any_change = False
new_cache = {}

file_path = "{{cwd}}"
sys.path.insert(0, file_path)

new_cache = {}
# cache filename should be dynamic
try:
    with open("{{bin_name}}.cache") as f:
        cache = json.load(f)
except (FileNotFoundError, json.JSONDecodeError):
    cache = {}

if use_timing:
    print("timing cache load", time.time() - sys.cliche_ts__)

# this path should be dynamic
for x in glob.glob("{{cwd}}/*.py") + glob.glob("{{cwd}}/**/*.py"):
    if any(e in x for e in ["#", "flycheck", "swp"]):
        continue
    mod_date = os.stat(x)[8]
    if x in cache:
        new_cache[x] = cache[x]
        if cache[x]["mod_date"] == mod_date:
            continue
    any_change = True
    with open(x) as f:
        contents = f.read()
        functions = re.findall(r"^ *@cli *\n *def ([^( ]+)+", contents, re.M)
        version = re.findall("""^ *__version__ = ['"]([^'"]+)""", contents)
        cache[x] = {
            "mod_date": mod_date,
            "functions": functions,
            "filename": x,
            "import_name": x.replace(file_path, "").strip("/").replace("/", ".").replace(".py", ""),
        }
        if version:
            cache[x]["version_info"] = version[0]
        new_cache[x] = cache[x]

if use_timing:
    print("timing cache build", time.time() - sys.cliche_ts__)

if any_change:
    cache = new_cache
    with open("{{bin_name}}.cache", "w") as f:
        json.dump(cache, f)

function_to_imports = {}
version_info = None
for cache_value in cache.values():
    import_name = cache_value["import_name"]
    functions = cache_value["functions"]
    version_info = version_info or cache_value.get("version_info")
    if not functions:
        continue
    for function in functions:
        function_to_imports[function] = import_name

if use_timing:
    print("timing function build", time.time() - sys.cliche_ts__)


def fallback(version_info=None):
    if use_timing:
        print("before imports", time.time() - sys.cliche_ts__)
    for import_name in sorted(set(function_to_imports.values())):
        t1 = time.time()
        __import__(import_name)
        if use_timing:
            print("import time", import_name, time.time() - sys.cliche_ts__)
    if use_timing:
        print("before main import", time.time() - sys.cliche_ts__)
    from cliche import main

    main(version_info=version_info)


if len(sys.argv) > 1:
    command = sys.argv[1].replace("-", "_")
    if command in function_to_imports:
        __import__(function_to_imports[command])
        if use_timing:
            print("before main import", time.time() - sys.cliche_ts__)
        from cliche import main

        main(version_info=version_info)
    else:
        if use_timing:
            print("before fallback", time.time() - sys.cliche_ts__)
        fallback(version_info=version_info)
else:
    if use_timing:
        print("before fallback", time.time() - sys.cliche_ts__)
    fallback(version_info=version_info)

if use_timing:
    print("kk", time.time() - sys.cliche_ts__)



from setuptools import find_packages
from setuptools import setup

MAJOR_VERSION = '0'
MINOR_VERSION = '7'
MICRO_VERSION = '60'
VERSION = "{}.{}.{}".format(MAJOR_VERSION, MINOR_VERSION, MICRO_VERSION)

setup(
    name='cliche',
    version=VERSION,
    description="A minimalistic CLI wrapper out to be the best",
    url='https://github.com/kootenpv/cliche',
    author='Pascal van Kooten',
    author_email='kootenpv@gmail.com',
    entry_points={'console_scripts': ['cliche = cliche.__init__:main']},
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    # install_requires=["argparse>=1.4.0"],
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: Customer Service',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: Microsoft',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Software Development',
        'Topic :: Software Development :: Build Tools',
        'Topic :: Software Development :: Debuggers',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Software Distribution',
        'Topic :: System :: Systems Administration',
        'Topic :: Utilities',
    ],
    zip_safe=False,
    platforms='any',
)
